/**
 * DynamicReports - Free Java reporting library for creating reports dynamically
 *
 * Copyright (C) 2010 - 2012 Ricardo Mariaca
 * http://dynamicreports.sourceforge.net
 *
 * This file is part of DynamicReports.
 *
 * DynamicReports is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DynamicReports is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with DynamicReports. If not, see <http://www.gnu.org/licenses/>.
 */

package net.sf.dynamicreports.test.jasper.datasource;

import static net.sf.dynamicreports.report.builder.DynamicReports.*;

import java.io.Serializable;

import net.sf.dynamicreports.jasper.builder.JasperReportBuilder;
import net.sf.dynamicreports.report.base.expression.AbstractSimpleExpression;
import net.sf.dynamicreports.report.builder.DatasetBuilder;
import net.sf.dynamicreports.report.builder.DynamicReports;
import net.sf.dynamicreports.report.builder.column.TextColumnBuilder;
import net.sf.dynamicreports.report.builder.crosstab.CrosstabBuilder;
import net.sf.dynamicreports.report.builder.crosstab.CrosstabColumnGroupBuilder;
import net.sf.dynamicreports.report.builder.crosstab.CrosstabMeasureBuilder;
import net.sf.dynamicreports.report.builder.crosstab.CrosstabRowGroupBuilder;
import net.sf.dynamicreports.report.constant.Calculation;
import net.sf.dynamicreports.report.datasource.DRDataSource;
import net.sf.dynamicreports.report.definition.ReportParameters;
import net.sf.dynamicreports.test.jasper.AbstractJasperCrosstabValueTest;
import net.sf.jasperreports.engine.JRDataSource;

/**
 * @author Ricardo Mariaca (dynamicreports@gmail.com)
 */
public class FilterTest extends AbstractJasperCrosstabValueTest implements Serializable {
	private static final long serialVersionUID = 1L;

	private TextColumnBuilder<String> column1;
	private TextColumnBuilder<String> column2;

	private CrosstabRowGroupBuilder<String> rowGroup;
	private CrosstabColumnGroupBuilder<String> columnGroup;
	private CrosstabMeasureBuilder<Integer> measure1;

	@Override
	protected void configureReport(JasperReportBuilder rb) {
		measure1 = ctab.measure("field3", Integer.class, Calculation.SUM);

		DatasetBuilder dataset = DynamicReports.dataset();
		dataset.setDataSource(createCrosstabDataSource());
		dataset.setFilterExpression(new CrosstabFilterExpression());

		CrosstabBuilder crosstab = ctab.crosstab()
			.setSubDataset(dataset)
			.rowGroups(
				rowGroup = ctab.rowGroup("field1", String.class))
			.columnGroups(
				columnGroup = ctab.columnGroup("field2", String.class))
			.measures(
				measure1);

		rb.columns(
				column1 = col.column("Column1", "field1", type.stringType()),
				column2 = col.column("Column2", "field2", type.stringType()))
			.title(crosstab)
			.setFilterExpression(new FilterExpression());
	}

	@Override
	public void test() {
		super.test();

		numberOfPagesTest(1);

		//column1
		columnTitleCountTest(column1, 1);
		columnTitleValueTest(column1, "Column1");
		columnDetailCountTest(column1, 1);
		columnDetailValueTest(column1, "1");
		//column2
		columnTitleCountTest(column2, 1);
		columnTitleValueTest(column2, "Column2");
		columnDetailCountTest(column2, 1);
		columnDetailValueTest(column2, "text1");

		setCrosstabBand("title");

		//column group
		crosstabGroupHeaderCountTest(columnGroup, 2);
		crosstabGroupHeaderValueTest(columnGroup, "c", "d");
		crosstabGroupTotalHeaderCountTest(columnGroup, 1);
		crosstabGroupTotalHeaderValueTest(columnGroup, "Total");

		//row group
		crosstabGroupHeaderCountTest(rowGroup, 1);
		crosstabGroupHeaderValueTest(rowGroup, "a");
		crosstabGroupTotalHeaderCountTest(rowGroup, 1);
		crosstabGroupTotalHeaderValueTest(rowGroup, "Total");

		//measure1
		crosstabCellCountTest(measure1, null, null, 2);
		crosstabCellValueTest(measure1, null, null, "3", "7");
		crosstabCellCountTest(measure1, null, columnGroup, 1);
		crosstabCellValueTest(measure1, null, columnGroup, "10");
		crosstabCellCountTest(measure1, rowGroup, null, 2);
		crosstabCellValueTest(measure1, rowGroup, null, "3", "7");
		crosstabCellCountTest(measure1, rowGroup, columnGroup, 1);
		crosstabCellValueTest(measure1, rowGroup, columnGroup, "10");
	}

	private class FilterExpression extends AbstractSimpleExpression<Boolean> {
		private static final long serialVersionUID = 1L;

		@Override
		public Boolean evaluate(ReportParameters reportParameters) {
			String value = reportParameters.getValue("field1");
			return value.equals("1");
		}
	}

	private class CrosstabFilterExpression extends AbstractSimpleExpression<Boolean> {
		private static final long serialVersionUID = 1L;

		@Override
		public Boolean evaluate(ReportParameters reportParameters) {
			String value = reportParameters.getValue("field1");
			return value.equals("a");
		}
	}

	@Override
	protected JRDataSource createDataSource() {
		DRDataSource dataSource = new DRDataSource("field1", "field2");
		dataSource.add("1", "text1");
		dataSource.add("2", "text2");
		dataSource.add("3", "text3");
		dataSource.add("4", "text4");
		return dataSource;
	}

	private JRDataSource createCrosstabDataSource() {
		DRDataSource dataSource = new DRDataSource("field1", "field2", "field3");
		dataSource.add("a", "c", 1);
		dataSource.add("a", "c", 2);
		dataSource.add("a", "d", 3);
		dataSource.add("a", "d", 4);
		dataSource.add("b", "c", 5);
		dataSource.add("b", "c", 6);
		dataSource.add("b", "d", 7);
		dataSource.add("b", "d", 8);
		return dataSource;
	}
}
