/**
 * DynamicReports - Free Java reporting library for creating reports dynamically
 *
 * Copyright (C) 2010 - 2012 Ricardo Mariaca
 * http://dynamicreports.sourceforge.net
 *
 * This file is part of DynamicReports.
 *
 * DynamicReports is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * DynamicReports is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with DynamicReports. If not, see <http://www.gnu.org/licenses/>.
 */

package net.sf.dynamicreports.test.jasper.subtotal;

import static net.sf.dynamicreports.report.builder.DynamicReports.*;

import java.util.Locale;

import net.sf.dynamicreports.jasper.builder.JasperReportBuilder;
import net.sf.dynamicreports.report.builder.column.TextColumnBuilder;
import net.sf.dynamicreports.report.builder.group.ColumnGroupBuilder;
import net.sf.dynamicreports.report.builder.subtotal.PercentageSubtotalBuilder;
import net.sf.dynamicreports.report.datasource.DRDataSource;
import net.sf.dynamicreports.test.jasper.AbstractJasperValueTest;
import net.sf.jasperreports.engine.JRDataSource;

/**
 * @author Ricardo Mariaca (dynamicreports@gmail.com)
 */
public class PercentageSubtotalTest extends AbstractJasperValueTest {
	private PercentageSubtotalBuilder subtotal1;
	
	@Override
	protected void configureReport(JasperReportBuilder rb) {		
		TextColumnBuilder<String> column1;
		TextColumnBuilder<Integer> column2;
		ColumnGroupBuilder group1;		
		
		rb.setLocale(Locale.ENGLISH)
			.columns(
					column1 = col.column("Column1", "field1", String.class),
					column2 = col.column("Column2", "field2", Integer.class))
			.groupBy(
					group1 = grp.group(column1))				
			.subtotalsOfPercentageAtGroupFooter(group1,
					subtotal1 = sbt.percentage(column2));
	}

	@Override
	public void test() {
		super.test();
		
		numberOfPagesTest(1);		
		//groupFooter
		subtotalCountTest(subtotal1, 2);
		subtotalValueTest(subtotal1, "28.57%", "71.43%");
	}
	
	@Override
	protected JRDataSource createDataSource() {
		DRDataSource dataSource = new DRDataSource("field1", "field2");
		for (int i = 1; i <= 3; i++) {
			dataSource.add("group1", i);
		}
		for (int i = 4; i <= 6; i++) {
			dataSource.add("group2", i);
		}	
		return dataSource;
	}
}
